/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include "dialogwarden.h"
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qsettings.h>
#include <QTextStream>



#include "dialogworking.h"
#include "dialogconfig.h"
#include "dialogcreatejail.h"
#include "dialogdisplayoutput.h"
#include "dialogimport.h"
#include "dialogsetupuser.h"

/* Update interval in ms  - Every 2 seconds */
#define  UPDATE_MSEC 1000 * 2

void dialogWarden::helpAbout()
{

}


void dialogWarden::programInit()
{
    // Setup our listview options
    listJails->setSelectionMode(QAbstractItemView::SingleSelection);
    listJails->setAllColumnsShowFocus(TRUE);
    
    // Read in our configuration data
    readConfig();
    
   // Start by refreshing our jail list
    refreshJails();
    
    // Start our monitor service to check if jails are running
    checkingStatus = false;
    currentStatusWorkingJail = "";
    statusTimer = new QTimer(this);
    connect( statusTimer, SIGNAL(timeout()), this, SLOT( slotMonitorJailStatus() ) );
    slotMonitorJailStatus();
    
    
    // Connect the rightclick slot 
    listJails->setContextMenuPolicy( Qt::CustomContextMenu);
    connect( listJails, SIGNAL(customContextMenuRequested (const QPoint & ) ), this, SLOT(slotJailRightClicked() ) );

}


void dialogWarden::refreshJails()
{
    QString AutoStart;
    
   listJails->clear();
   
   // Check our JailDir and see what we have
   QDir d(JailDir);
   d.setFilter( QDir::Dirs);

   for ( int i = 0; i < d.count(); i++ ) {
       
       // Traverse our directories, looking for jails
       if ( d[i] != "." && d[i] != ".." ) {
	   
            // Check for the hostname of this jail
            QFile file( JailDir + "/" + d[i] + "/etc/.wardenhost" );
            if ( file.exists() ) {
	if ( file.open( QIODevice::ReadOnly ) ) {
	    QTextStream stream( &file );
	    QString host;
        
	    while ( !stream.atEnd() ) {
		host = stream.readLine(); // line of text excluding '\n'
	    }
	    
	          // Check if this jail is set to autostart
	          AutoStart = "Disabled";
	          QFile file2(JailDir + "/" + d[i] + "/etc/.wardenautostart" );
	          if ( file2.exists() )
	          {
		     AutoStart = "Enabled";  
	          }
	           QStringList cols;
                   cols << d[i] << host << AutoStart << "<Unknown>"; 
	           (void) new QTreeWidgetItem( listJails, cols );
                  }
	}
        }
   
   }
   

   
}


void dialogWarden::readConfig()
{  
    
    QSettings settings("PCBSD");

    ProgDir = settings.value( "/PC-BSD/TheWarden/ProgDir", WARDENDIR ).toString();
    WorldSrc = settings.value( "/PC-BSD/TheWarden/WorldSrc", QString("/usr/src") ).toString();
    WorldDir = settings.value( "/PC-BSD/TheWarden/WorldDir", WARDENDIR + QString("/worlds/buildworld") ).toString();
    NIC = settings.value( "/PC-BSD/TheWarden/NIC", "UNSET" ).toString();
    TMPDIR = settings.value( "/PC-BSD/TheWarden/TMPDIR", WARDENDIR + QString("/tmp")).toString();
    JailDir = settings.value( "/PC-BSD/TheWarden/JailDir", WARDENDIR + QString("/jails") ).toString();
    
}


void dialogWarden::slotOpenConfig()
{
        // Launch our AddPartitionDialog to add a new device
        configDialog = new dialogConfig();
        connect(configDialog, SIGNAL(saved()),this, SLOT(readConfig()) );
        configDialog->programInit();
        configDialog->exec();

}


void dialogWarden::slotExit()
{
    exit(0);
}


void dialogWarden::slotCheckJailStatus()
{
    QString oldWorking = currentStatusWorkingJail;
    int found = 0;
    // Mark that we are in the middle of checking the status
    checkingStatus = true;
    
    // Read in our iterator and start checking jail status
    QTreeWidgetItemIterator it( listJails );
    while (*it) {
	
	if ( currentStatusWorkingJail.isEmpty() ) {
	    currentStatusWorkingJail = (*it)->text(0);
	    break;
	}
	
	if ( found == 1)
	{
	  currentStatusWorkingJail = (*it)->text(0);
	  break;
	}
	
	if ( currentStatusWorkingJail == (*it)->text(0) )
	{
	    found = 1;
	}
	
	
        ++it;
    }
    
    
    // If we have a jail to check status on, do it
    if ( currentStatusWorkingJail == oldWorking ) {
	checkingStatus = false;
	currentStatusWorkingJail="";
	return;
    }
      // Run the getUpdatesDir.sh script to rsync with master server
      statusProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/checkstatus.sh";
      QStringList args;
      args << currentStatusWorkingJail;
      
      // Connect the exited signal and start the process
      connect( statusProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotCheckStatusReturn() ) );
      statusProc->start(program, args);
}


void dialogWarden::slotMonitorJailStatus()
{
    if ( checkingStatus == true) {
       // Set our timer to check again
       statusTimer->start( UPDATE_MSEC );
  } else {
      currentStatusWorkingJail = "";
      slotCheckJailStatus();
      // Set our timer to check again
      statusTimer->start( UPDATE_MSEC );
  }

}


void dialogWarden::slotCheckStatusReturn()
{

    // Check the exit status
    int exitStatus = statusProc->exitCode();
    
        // Read in our iterator and locate the jail, changing its status
    QTreeWidgetItemIterator it( listJails );
    while (*it) {
	if ( currentStatusWorkingJail == (*it)->text(0) )
	{
	    if ( exitStatus == 0) {
		(*it)->setText(3, tr("Running"));
	    } else {
		(*it)->setText(3, tr("Stopped"));
	    }
	    
	    break;
	}
	
	
        ++it;
    }
    
    // Check the next jail, if there are any more left. 
    slotCheckJailStatus();
}




void dialogWarden::slotJailRightClicked()
{
    QString Status;
    bool ok;

    if ( listJails->currentItem()) {
          popupip = listJails->currentItem()->text(0);
          Status = listJails->currentItem()->text(3);
	  if ( Status == tr("Running") ) {
	      popup = new QMenu();
	      popup->addAction( tr("Stop this Jail") , this, SLOT(slotStopJail() )  );
	      popup->addSeparator();
	      popup->addAction( tr("Toggle Autostart") , this, SLOT(slotToggleAutostartClicked() )  );
	      popup->addAction( tr("Install Inmate into jail") , this, SLOT(slotInmateInstallClicked() )  );
	      popup->addAction( tr("View installed packages") , this, SLOT(slotListJailPkgs() )  );
	      popup->addAction( tr("Export jail to .wdn file") , this, SLOT(slotExportJail() )  );
	      popup->addSeparator();
	      popup->addAction( tr("Delete Jail") , this, SLOT(slotDeleteJail() )  );
	      popup->exec( QCursor::pos() );
	  } else {
	      popup = new QMenu();
	      popup->addAction( tr("Start this Jail") , this, SLOT(slotStartJail() )  );
	      popup->addSeparator();
	      popup->addAction( tr("Toggle Autostart") , this, SLOT(slotToggleAutostartClicked() )  );
	      popup->addAction( tr("Install Inmate into jail") , this, SLOT(slotInmateInstallClicked() )  );
	      popup->addAction( tr("View installed packages") , this, SLOT(slotListJailPkgs() )  );
	      popup->addAction( tr("Export jail to .wdn file") , this, SLOT(slotExportJail() )  );
	      popup->addSeparator();
	      popup->addAction( tr("Delete Jail") , this, SLOT(slotDeleteJail() )  );
	      popup->exec( QCursor::pos() );
	  }


    }

    
}


void dialogWarden::slotStopJail()
{
        QString tmp;
	
        // Launch our working dialog to prevent further action until we are finished
        workingDialog = new dialogWorking();
        workingDialog->setModal(true);
        workingDialog->programInit();
        tmp = tr("Stopping Jail");
        workingDialog->setDialogTitle(tmp);
        tmp = tr("Stopping Jail:") + " " + popupip;
        workingDialog->setDialogText(tmp);
        workingDialog->show();
	
	
      // Now start the script to stop this jail
      stopJailProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/stopjail.sh";
      QStringList args;
      args << popupip;
      
      // Connect the exited signal and start the process
      connect( stopJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedWorking() ) );
      stopJailProc->start(program, args);
	
}


void dialogWarden::slotStartJail()
{
        QString tmp;
	
        // Launch our working dialog to prevent further action until we are finished
        workingDialog = new dialogWorking();
        workingDialog->setModal(true);
        workingDialog->programInit();
        tmp = tr("Starting Jail");
        workingDialog->setDialogTitle(tmp);
        tmp = tr("Starting Jail:") + " " + popupip;
        workingDialog->setDialogText(tmp);
        workingDialog->show();
	
	
      // Now start the script to stop this jail
      startJailProc = new QProcess( this );
      QString program =  ProgDir + "/scripts/backend/startjail.sh";
      QStringList args;
      args << popupip;
      
      // Connect the exited signal and start the process
      connect( startJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedWorking() ) );
      startJailProc->start(program, args);
}



void dialogWarden::slotExportJail()
{
    QString exportFile, tmp;
    
    exportFile = QFileDialog::getExistingDirectory(
                    this,
		    tr("Choose a directory to save the finished .wdn file:"),
                    "/home",
                    QFileDialog::ShowDirsOnly );
    
    if ( exportFile.isEmpty() )
    {
	return;
    }
    
      // Lauch the command output dialog
    /*
      dialogOutput = new dialogDisplayOutput(0, "Exporting Jail", Qt::WStyle_Customize | Qt::WShowModal);
      dialogOutput->programInit(FALSE);
      dialogOutput->setDialogCaption("Exporting Jail: " + popupip);
      dialogOutput->setDialogText("");
      dialogOutput->show();
      */

            // Launch our working dialog to prevent further action until we are finished
      workingDialog = new dialogWorking();
      workingDialog->setModal(true);
      workingDialog->programInit();
      tmp = tr("Exporting Jail");
      workingDialog->setDialogTitle(tmp);
      tmp = tr("Exporting Jail:") + " " + popupip;
      workingDialog->setDialogText(tmp);
      workingDialog->show();
    
          // Now start the script to stop this jail
      exportJailProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/exportjail.sh";
      QStringList args;
      args << popupip << exportFile;
      
      // Connect the exited signal and start the process 
      // connect( exportJailProc, SIGNAL(readyReadStdout ()), this, SLOT(slotReadDisplayOutput() ) );
      connect( exportJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedWorking() ) );
      exportJailProc->start(program, args);
   
}


void dialogWarden::slotListJailPkgs()
{

    
      dialogOutput = new dialogDisplayOutput();
      dialogOutput->setModal(true);
      dialogOutput->programInit(TRUE);
      dialogOutput->setDialogCaption("Jail Packages: " + popupip);
      dialogOutput->setDialogText("");
      dialogOutput->show();
    
          // Now start the script to stop this jail
      listPackagesProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/listpkgs.sh";
      QStringList args;
      args << popupip;

      listPackagesProc->setProcessChannelMode(QProcess::MergedChannels);
      listPackagesProc->setReadChannel(QProcess::StandardOutput);

      // Connect the exited signal and start the process 
      connect( listPackagesProc, SIGNAL(readyReadStandardOutput ()), this, SLOT(slotReadPkgsOutput() ) );

      listPackagesProc->start(program, args);
}


void dialogWarden::slotDeleteJail()
{
    QString tmp;
    
       // Launch our working dialog to prevent further action until we are finished
      workingDialog = new dialogWorking();
      workingDialog->setModal(true);
      workingDialog->programInit();
      tmp = tr("Deleting Jail");
      workingDialog->setDialogTitle(tmp);
      tmp = tr("Deleting Jail:") + " " + popupip;
      workingDialog->setDialogText(tmp);
      workingDialog->show();
    
          // Now start the script to stop this jail
      deleteJailProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/deletejail.sh";
      QStringList args;
      args << popupip;
      
      // Connect the exited signal and start the process 
      connect( deleteJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(refreshJails() ) );
      connect( deleteJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedWorking() ) );
      deleteJailProc->start(program, args);
}




void dialogWarden::slotFinishedWorking()
{
        // Finished whatever we were working on, hide the dialog now. 
        workingDialog->hide();
}


void dialogWarden::runCommand( QString command )
{
        FILE *file = popen(command.toLatin1(),"r");
        fclose(file);
}


void dialogWarden::slotReadDisplayOutput()
{
    
    
    while ( exportJailProc->canReadLine() )
    {
	dialogOutput->appendDialogText(exportJailProc->readLine());
    }

}


void dialogWarden::slotFinishedOutput()
{
    // Hide the output dialog, not needed anymore
    dialogOutput->hide();
}


void dialogWarden::slotReadPkgsOutput()
{
    
    while ( listPackagesProc->canReadLine() )
    {
	dialogOutput->appendDialogText(listPackagesProc->readLine());
    }
}



void dialogWarden::slotClickedNewJail()
{
       // Launch our working dialog to prevent further action until we are finished
      newJailDialog = new dialogCreateJail();
      newJailDialog->setModal(true);
      newJailDialog->programInit();
      
      connect( newJailDialog, SIGNAL(create(const QString &, const QString &, bool, bool, bool)), this, SLOT(slotCreateNewJail(const QString &, const QString &, bool, bool, bool) ) );
      newJailDialog->show();
            
      
              


}




void dialogWarden::slotCreateNewJail( const QString &IP, const QString &host, bool src, bool ports, bool autostart )
{
    // Save the provided variables so we can use them at creation time
    newIP = IP;
    newHost = host;
    newSrc = src;
    newPorts = ports;
    newAutostart = autostart;
    
    
    // First Prompt for the root password for this jail
      userDialog = new dialogSetupUser();
      userDialog->setModal(true);
      userDialog->programInit();
      connect( userDialog, SIGNAL(saved(const QString &, const QString &, const QString &)), this, SLOT(slotSavedUserSettings(const QString &, const QString &, const QString &) ) );
      userDialog->show();       
}


void dialogWarden::slotReadCreateOutput()
{
    while ( createJailProc->canReadLine() )
    {
	dialogOutput->appendDialogText(createJailProc->readLine());
    }
}


void dialogWarden::slotFinishedJailCreate()
{
    // Jail is done creating, now add the username to it
      userAddProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/setupusers.sh";
      QStringList args;
      args << newIP << newRootPW << newUser << newUserPW;
      
      // Connect the exited signal and start the process 
      userAddProc->setProcessChannelMode(QProcess::MergedChannels);
      userAddProc->setReadChannel(QProcess::StandardOutput);
      connect( userAddProc, SIGNAL(readyReadStandardOutput()), this, SLOT(slotReadUserAddOutput() ) );
      connect( userAddProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedUserAdd() ) );
      userAddProc->start(program, args);
    

}


void dialogWarden::slotSavedUserSettings( const QString &root, const QString &user, const QString &userpw )
{
    // Save our variables
    newRootPW = root;
    newUser = user;
    newUserPW = userpw;
    
    
    /***********************************************************************************************************************
        Now create the jail here
    *************************************************************************************************************************/
      dialogOutput = new dialogDisplayOutput();
      dialogOutput->setModal(true);
      dialogOutput->programInit(FALSE);
      dialogOutput->setDialogCaption("Creating Jail: " + newIP);
      dialogOutput->setDialogText("");
      dialogOutput->show();
    
    
      // Now start the script to stop this jail
      createJailProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/createjail.sh";
      QStringList args;
      args <<  newIP << newHost;

      // Set our flags if we want source / ports tree or not
      if ( newSrc )
	args << "YES";
      else 
	args << "NO";

      if ( newPorts )
	args <<  "YES";
      else 
	args << "NO";

      if ( newAutostart )
	args << "YES";
      else
	args << "NO";

      
      // Connect the exited signal and start the process 
      createJailProc->setProcessChannelMode(QProcess::MergedChannels);
      createJailProc->setReadChannel(QProcess::StandardOutput);
      connect( createJailProc, SIGNAL(readyReadStandardOutput ()), this, SLOT(slotReadCreateOutput() ) );
      connect( createJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedJailCreate() ) );
      createJailProc->start(program, args);

}


void dialogWarden::slotReadUserAddOutput()
{
    while ( userAddProc->canReadLine() )
    {
	dialogOutput->appendDialogText(userAddProc->readLine());
    }
}


void dialogWarden::slotFinishedUserAdd()
{
    // All finished with creating  a jail and adding a username to it
    
    // Unhide the close button
    dialogOutput->setCloseHide(FALSE);
    
       // Refresh our jail list
    refreshJails();
}


void dialogWarden::slotImportJailClicked()
{
      // The user wants to import a jail, lets prompt for the .wdn file to use
      importFile="";
    
      importFile = QFileDialog::getOpenFileName(
                    this,
                    tr("Select warden jail package"),
                    "/home",
                    "Warden Jail Packs (*.wdn)");
      
      if ( importFile.isEmpty() )
	  return;

      // Bring up dialog to ask about import settings
      importDialog = new dialogImport();
      importDialog->programInit();
      connect( importDialog, SIGNAL(import(const QString &, const QString &)), this, SLOT(slotImportConfigDone(const QString &, const QString &) ) );
      importDialog->show();

}

void dialogWarden::slotImportConfigDone(const QString &IP, const QString &Host)
{

      // Display the import dialog output
      dialogOutput = new dialogDisplayOutput();
      dialogOutput->setModal(true);
      dialogOutput->programInit(FALSE);
      dialogOutput->setDialogCaption("Importing Jail");
      dialogOutput->setDialogText("");
      dialogOutput->show();

      importJailProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/importjail.sh";
      QStringList args;
      args << importFile << IP << Host;

      
      // Connect the exited signal and start the process 
      importJailProc->setProcessChannelMode(QProcess::MergedChannels);
      importJailProc->setReadChannel(QProcess::StandardOutput);
      connect( importJailProc, SIGNAL(readyReadStandardOutput ()), this, SLOT(slotReadImportOutput() ) );
      connect( importJailProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedJailImport()));
      importJailProc->start(program, args);

}


void dialogWarden::slotReadImportOutput()
{
    while ( importJailProc->canReadLine() )
    {
	dialogOutput->appendDialogText(importJailProc->readLine());
    }
}


void dialogWarden::slotFinishedJailImport()
{
    // All finished with importing a jail
    
    // Unhide the close button
    dialogOutput->setCloseHide(FALSE);
    
       // Refresh our jail list
    refreshJails();
}


void dialogWarden::slotInmateInstallClicked()
{
      // The user wants to install an inmate
      importFile="";
    
      importFile = QFileDialog::getOpenFileName(
		    this,
                    tr("open file dialog"),
                    "/home",
                    "Warden Inmate Packs (*.wit)");
      
      if ( importFile.isEmpty() )
      {
	  return;
      }
      
      // Display the import dialog output
      dialogOutput = new dialogDisplayOutput();
      dialogOutput->setModal(true);
      dialogOutput->programInit(FALSE);
      dialogOutput->setDialogCaption("Installing Inmate in: " + popupip);
      dialogOutput->setDialogText("");
      dialogOutput->show();
      
      
      loadInmateProc = new QProcess( this );
      QString program = ProgDir + "/scripts/backend/installinmate.sh";
      QStringList args;
      args << popupip << importFile;

      
      // Connect the exited signal and start the process 
      loadInmateProc->setProcessChannelMode(QProcess::MergedChannels);
      loadInmateProc->setReadChannel(QProcess::StandardOutput);
      connect( loadInmateProc, SIGNAL(readyReadStandardOutput ()), this, SLOT(slotReadInmateOutput() ) );
      connect( loadInmateProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotFinishedInmateInstall() ) );
      loadInmateProc->start(program, args);
      
}


void dialogWarden::slotReadInmateOutput()
{
    while ( loadInmateProc->canReadLine() )
    {
	dialogOutput->appendDialogText(loadInmateProc->readLine());
    }
}


void dialogWarden::slotFinishedInmateInstall()
{
    // All finished with importing a jail
    
    // Unhide the close button
    dialogOutput->setCloseHide(FALSE);
    
       // Refresh our jail list
    refreshJails();
}


void dialogWarden::slotToggleAutostartClicked()
{
    // Change auto-start status of the jail
    //runCommand("warden auto " + popupip);
    
    toggleAutoProc = new QProcess( this );
    QString program = "warden";
    QStringList args;
    args << "auto" << popupip;

      
    // Connect the exited signal and start the process 
    connect( toggleAutoProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(refreshJails() ) );
    toggleAutoProc->start(program, args);
    
}
